@@ -1,5 +1,5 @@
 from gdeltdoc.api_client import GdeltDoc
-from gdeltdoc.filters import Filters, near, repeat, multi_repeat, VALID_TIMESPAN_UNITS
+from gdeltdoc.filters import Filters, near, multi_near, repeat, multi_repeat, VALID_TIMESPAN_UNITS
 from gdeltdoc._version import version
 
 __version__ = version
