@@ -1,10 +1,19 @@
 from typing import Optional, List, Union, Tuple
+
+try:
+    from typing import Unpack
+except ImportError:
+    from typing_extensions import Unpack
+
+
 from string import ascii_lowercase, digits
+from gdeltdoc.validation import validate_tone
 
 Filter = Union[List[str], str]
 
 VALID_TIMESPAN_UNITS = ["min", "h", "hours", "d", "days", "w", "weeks", "m", "months"]
 
+
 def near(n: int, *args) -> str:
     """
     Build the filter to find articles containing words that occur within
@@ -19,6 +28,26 @@ def near(n: int, *args) -> str:
     return f"near{str(n)}:" + '"' + " ".join([a for a in args]) + '" '
 
 
+def multi_near(
+    nears: List[Tuple[int, Unpack[Tuple[str, ...]]]], method: str = "OR"
+) -> str:
+    """
+    Build the filter to find articles containing multiple sets of near terms.
+
+    eg. multi_near([(5, "airline", "crisis"), (10, "airline", "climate", "change")]) finds "airline" and "crisis"
+    within 5 words, and/or "airline", "climate", and "change" within 10 words
+    """
+    if method not in ["AND", "OR"]:
+        raise ValueError(f"method must be one of AND or OR, not {method}")
+
+    formatted = [near(n, *args) for (n, *args) in nears]
+
+    paren_flag = len(formatted) != 1 and method == "OR"
+    l_pad, r_pad = paren_flag * "(", paren_flag * ") "
+
+    return l_pad + f"{method} ".join(formatted) + r_pad
+
+
 def repeat(n: int, keyword: str) -> str:
     """
     Build the filter to find articles containing `keyword` at least `n` times.
@@ -53,7 +82,8 @@ def multi_repeat(repeats: List[Tuple[int, str]], method: str) -> str:
 
     if method == "AND":
         return f"{method} ".join(to_repeat)
-    elif method == "OR":
+    else:
+        # method == "OR"
         return "(" + f"{method} ".join(to_repeat) + ")"
 
 
@@ -70,7 +100,10 @@ class Filters:
         near: Optional[str] = None,
         repeat: Optional[str] = None,
         country: Optional[Filter] = None,
+        language: Optional[Filter] = None,
         theme: Optional[Filter] = None,
+        tone: Optional[Filter] = None,
+        tone_absolute: Optional[Filter] = None,
     ) -> None:
         """
         Construct filters for the GDELT API.
@@ -105,6 +138,15 @@ class Filters:
             Return articles from the specified domain. Does not require an exact match so
             passing "cnn.com" will match articles from "cnn.com", "subdomain.cnn.com" and "notactuallycnn.com".
 
+            NOTE: There's a bug in the API when filtering for domains that include a dash (-).
+            If you supply a value for this filter using a domain with a dash you'll always get 0 results.
+
+            As a workaround you can filter using the part of the domain after the dash (eg. cdn-apple.com -> apple.com).
+            The query is an inexact match so this will return results from the domain you want, plus other domains
+            which also match. You'll need to filter the results after you get them back to just the single domain.
+
+            For more information on this bug, please see https://github.com/alex9smith/gdelt-doc-api/issues/35
+
         domain_exact
             Similar to `domain`, but requires an exact match.
 
@@ -121,9 +163,22 @@ class Filters:
         country
             Return articles published in a country, formatted as the FIPS 2 letter country code.
 
+        language
+            Return articles published in the given language, formatted as the ISO 639 language code.
+
         theme
             Return articles that cover one of GDELT's GKG Themes. A full list of themes can be
             found here: http://data.gdeltproject.org/api/v2/guides/LOOKUP-GKGTHEMES.TXT
+
+        tone
+            Return articles above or below a particular tone score (ie more positive or more negative than
+            a certain threshold). To use, specify either a greater than or less than sign and a positive
+            or negative number (either an integer or floating point number). To find fairly positive
+            articles, search for "tone>5" or to search for fairly negative articles, search for "tone<-5"
+
+        tone_absolute
+            The same as `tone` but ignores the positive/negative sign and lets you simply search for
+            high emotion or low emotion articles, regardless of whether they were happy or sad in tone
         """
         self.query_params: List[str] = []
         self._valid_countries: List[str] = []
@@ -150,9 +205,20 @@ class Filters:
         if country:
             self.query_params.append(self._filter_to_string("sourcecountry", country))
 
+        if language:
+            self.query_params.append(self._filter_to_string("sourcelang", language))
+
         if theme:
             self.query_params.append(self._filter_to_string("theme", theme))
 
+        if tone:
+            validate_tone(tone)
+            self.query_params.append(self._tone_to_string("tone", tone))
+
+        if tone_absolute:
+            validate_tone(tone_absolute)
+            self.query_params.append(self._tone_to_string("toneabs", tone_absolute))
+
         if near:
             self.query_params.append(near)
 
@@ -160,12 +226,15 @@ class Filters:
             self.query_params.append(repeat)
 
         if start_date:
+            if not end_date:
+                raise ValueError("Must provide both start_date and end_date")
+
             self.query_params.append(
                 f'&startdatetime={start_date.replace("-", "")}000000'
             )
             self.query_params.append(f'&enddatetime={end_date.replace("-", "")}000000')
-        else:
-            # Use timespan
+
+        elif timespan:
             self._validate_timespan(timespan)
             self.query_params.append(f"&timespan={timespan}")
 
@@ -233,6 +302,31 @@ class Filters:
                 + ") "
             )
 
+    @staticmethod
+    def _tone_to_string(name: str, tone: Filter) -> str:
+        """
+        Convert a Filter for tone or tone_absolute into the string for the API.
+        Tone is different to the other parameters because it has no colon between
+        the API parameter and the value.
+
+        Params
+        ------
+        name
+            The parameter name as required by the API. Must be either `tone` or `toneabs`
+        tone
+            The tone Filter
+
+        Returns
+        -------
+        str
+            The converted filter eg. "tone>5"
+        """
+        if type(tone) == str:
+            return f"{name}{tone} "
+
+        else:
+            raise NotImplementedError("Multiple tone values are not supported yet.")
+
     @staticmethod
     def _validate_timespan(timespan: str) -> None:
         """
@@ -256,13 +350,19 @@ class Filters:
         """
 
         value = timespan.rstrip(ascii_lowercase)
-        unit = timespan[len(value):]
+        unit = timespan[len(value) :]
 
         if unit not in VALID_TIMESPAN_UNITS:
-            raise ValueError(f"Timespan {timespan} is invalid. {unit} is not a supported unit, must be one of {' '.join(VALID_TIMESPAN_UNITS)}")
+            raise ValueError(
+                f"Timespan {timespan} is invalid. {unit} is not a supported unit, must be one of {' '.join(VALID_TIMESPAN_UNITS)}"
+            )
 
         if not all(d in digits for d in value):
-            raise ValueError(f"Timespan {timespan} is invalid. {value} could not be converted into an integer")
+            raise ValueError(
+                f"Timespan {timespan} is invalid. {value} could not be converted into an integer"
+            )
 
         if unit == "min" and int(value) < 60:
-            raise ValueError(f"Timespan {timespan} is invalid. Period must be at least 60 minutes")
+            raise ValueError(
+                f"Timespan {timespan} is invalid. Period must be at least 60 minutes"
+            )
